<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2024 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2022.2/2545213
 */
namespace Projects\Checker;

use Application\Checker;
use Application\Config\ConfigManager as Cm;
use Application\Config\IConfigDefinition as IDef;
use Application\Config\ConfigException;
use Application\Config\Services;
use Application\Permissions\IPermissions;
use InvalidArgumentException;

/**
 * Checker for project functions
 */
class Project extends Checker
{
    /**
     * Check a project action
     * @throws ConfigException
     * @see Checker
     */
    public function check(string $check, array $options = null): bool
    {
        switch ($check) {
            case IProject::PROJECT_ADD:
                $this->checkAddProject();
                break;
            case IProject::PROJECT_NAME_ADMIN_ONLY:
                return $this->checkAdminForConfigSetting(IDef::PROJECTS_EDIT_NAME_ADMIN_ONLY);
            case IProject::BRANCHES_ADMIN_ONLY:
                return $this->checkAdminForConfigSetting(IDef::PROJECTS_EDIT_BRANCHES_ADMIN_ONLY);
            default:
                throw new InvalidArgumentException(sprintf("Check '%s' is not supported", $check));
        }
        return true;
    }

    /**
     * Check the add project action. Checks the admin only and groups only configuration values to determine if
     * add project is allowed based on the user role and their group membership
     * @throws ConfigException
     */
    private function checkAddProject()
    {
        $configCheck = $this->services->get(Services::CONFIG_CHECK);
        $config      = $this->services->get(IDef::CONFIG);
        $adminOnly   = Cm::getValue($config, IDef::PROJECTS_ADD_ADMIN_ONLY, false);
        if ($adminOnly) {
            $configCheck->check(IPermissions::ADMIN_CHECKER);
        }
        $addProjectGroups = Cm::getValue($config, IDef::PROJECTS_ADD_GROUPS_ONLY, []);
        if ($addProjectGroups) {
            $checker                   = IPermissions::MEMBER_CHECKER;
            $checker[Checker::OPTIONS] = [self::VALUE => $addProjectGroups];
            $configCheck->check($checker);
        }
    }
}
